﻿using System;
using System.Collections.Generic;
using System.Windows;
using LearningMEF.PluginLoader;
using LearningMEF.PluginLoader.Plugins;

namespace LearningMEF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            PluginManager = new PluginManager();
        }

        private void LoadPluginsButtons_Click(object sender, RoutedEventArgs e)
        {
            PluginManager.ComposePlugins();

            PopulateListWithPlugins(PluginManager.LoadedIPlugins);
        }

        private void RefreshPluginsButtons_Click(object sender, RoutedEventArgs e)
        {
            PluginManager.RefreshExternalPlugins();

            PopulateListWithPlugins(PluginManager.LoadedIPlugins);
        }

        private void PopulateListWithPlugins( List<Lazy<IPlugin, IPluginMetadata>> loadedIPlugins )
        {
            PluginManager.Logger.LogDebug( "Starting to clear the existing items" );
            LoadPluginsListBox.Items.Clear();

            foreach (var loadedIPlugin in loadedIPlugins)
            {
                LoadPluginsListBox.Items.Add(new ListBoxItem(loadedIPlugin));
            }
        }



        private PluginManager PluginManager { get; set; }

        private void RunPluginButton_Click(object sender, RoutedEventArgs e)
        {
            if (LoadPluginsListBox.SelectedItem == null)
            {
                MessageBox.Show("Kindly select an item prior to clicking me..... Thanks");
                return;
            }

            var selectedItem = (ListBoxItem)LoadPluginsListBox.SelectedItem;
            var actionResult = selectedItem.Plugin.Value.PluginAction();
            PluginResultTextBox.Text = actionResult;
        }

        private class ListBoxItem
        {
            public ListBoxItem( Lazy<IPlugin, IPluginMetadata> plugin )
            {
                Plugin = plugin;
            }

            public override string ToString()
            {
                return string.Format( "Plugin: Name {0} Version {1}",
                    Plugin.Metadata.Name, Plugin.Metadata.Version );
                //return "Plugin Class - " + Plugin.GetType().Name;
            }

            public Lazy<IPlugin, IPluginMetadata> Plugin { get; set; }
        }

    }
}
