﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.IO;using LearningMEF.PluginLoader.Plugins;

namespace LearningMEF.PluginLoader
{
    public class PluginManager
    {
        [ImportMany( typeof( IPlugin ), AllowRecomposition = true )]
        private List<Lazy<IPlugin, IPluginMetadata>> _loadedIPlugins = new List<Lazy<IPlugin, IPluginMetadata>>();

        public PluginManager()
        {
            CreateCatalog();
        }

        private AggregateCatalog _aggregateCatalog;
        private DirectoryCatalog _runtimeDirectoryCatalog;
        private DirectoryCatalog _pluginDirectoryCatalog;
        private CompositionContainer _compositionContainer;

        public void ComposePlugins()
        {
            var compositionBatch = new CompositionBatch();
            compositionBatch.AddPart( this );

            _aggregateCatalog = new AggregateCatalog();
            _aggregateCatalog.Catalogs.Add(_runtimeDirectoryCatalog);
            _aggregateCatalog.Catalogs.Add(_pluginDirectoryCatalog);

            _compositionContainer = new CompositionContainer(_aggregateCatalog);
            _compositionContainer.Compose( compositionBatch );
        }

        public void RefreshExternalPlugins()
        {
            _pluginDirectoryCatalog.Refresh();
        }

        private void CreateCatalog()
        {
            _runtimeDirectoryCatalog = new DirectoryCatalog( Environment.CurrentDirectory );

            var pluginFolder = Path.Combine( Environment.CurrentDirectory, "Plugins" );

            Directory.CreateDirectory( pluginFolder );

            _pluginDirectoryCatalog = new DirectoryCatalog( pluginFolder );
        }

        public List<Lazy<IPlugin, IPluginMetadata>> LoadedIPlugins { get { return _loadedIPlugins; } }

        public Logger Logger
        {
            get
            {
                var logger = _compositionContainer.GetExportedValueOrDefault<Logger>();

                if ( logger == null )
                {
                    return new DefaultLogger();
                }

                return logger;
            }
        }

    }
}
