﻿using System.Collections.Generic;
using System.Windows;
using LearningMEF.PluginLoader;
using LearningMEF.PluginLoader.Plugins;

namespace LearningMEF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            PluginManager = new PluginManager();
        }

        private void LoadPluginsButtons_Click(object sender, RoutedEventArgs e)
        {
            PluginManager.ComposePlugins();

            PopulateListWithPlugins(PluginManager.LoadedIPlugins);
        }

        private void RefreshPluginsButtons_Click(object sender, RoutedEventArgs e)
        {
            PluginManager.RefreshExternalPlugins();

            PopulateListWithPlugins(PluginManager.LoadedIPlugins);
        }

        private void PopulateListWithPlugins(List<IPlugin> loadedIPlugins)
        {
            LoadPluginsListBox.Items.Clear();

            foreach (var loadedIPlugin in loadedIPlugins)
            {
                LoadPluginsListBox.Items.Add(new ListBoxItem(loadedIPlugin));
            }
        }
        
        private PluginManager PluginManager { get; set; }

        private void RunPluginButton_Click(object sender, RoutedEventArgs e)
        {
            if (LoadPluginsListBox.SelectedItem == null)
            {
                MessageBox.Show("Kindly select an item prior to clicking me..... Thanks");
                return;
            }

            var selectedItem = (ListBoxItem)LoadPluginsListBox.SelectedItem;
            var actionResult = selectedItem.Plugin.PluginAction();
            PluginResultTextBox.Text = actionResult;
        }

        private class ListBoxItem
        {
            public ListBoxItem(IPlugin plugin)
            {
                Plugin = plugin;
            }

            public override string ToString()
            {
                return "Plugin Class - " + Plugin.GetType().Name;
            }

            public IPlugin Plugin { get; set; }
        }

    }
}
