﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.IO;
using System.Linq;
using System.Text;
using LearningMEF.PluginLoader.Plugins;

namespace LearningMEF.PluginLoader
{
    public class PluginManager
    {

        [ImportMany( typeof (IPlugin), AllowRecomposition = true )] 
        private List<IPlugin> _loadedIPlugins = new List<IPlugin>();

        public PluginManager()
        {
            CreateCatalog();
        }

        private AggregateCatalog _aggregateCatalog;
        private DirectoryCatalog _runtimeDirectoryCatalog;
        private DirectoryCatalog _pluginDirectoryCatalog;

        public void ComposePlugins()
        {
            var compositionBatch = new CompositionBatch();
            compositionBatch.AddPart( this );

            _aggregateCatalog = new AggregateCatalog();
            _aggregateCatalog.Catalogs.Add( _runtimeDirectoryCatalog );
            _aggregateCatalog.Catalogs.Add( _pluginDirectoryCatalog );

            var compositionContainer = new CompositionContainer( _aggregateCatalog );
            compositionContainer.Compose( compositionBatch );
        }

        public void RefreshExternalPlugins()
        {
            _pluginDirectoryCatalog.Refresh();
        }

        private void CreateCatalog()
        {
            _runtimeDirectoryCatalog = new DirectoryCatalog( Environment.CurrentDirectory );

            var pluginFolder = Path.Combine( Environment.CurrentDirectory, "Plugins" );

            Directory.CreateDirectory( pluginFolder );

            _pluginDirectoryCatalog = new DirectoryCatalog( pluginFolder );
        }

        public List<IPlugin> LoadedIPlugins
        {
            get { return _loadedIPlugins; }
        }

    }
}
